from typing import Literal
import base64
import numpy as np
import cv2 as cv
import numpy as np
import ramda as R

@R.curry
def encode(img: cv.Mat, format: Literal[".png", ".jpg", ".webp"]) -> bytes: 
    _, encoded = cv.imencode(format, img)
    return encoded.tobytes()

def decode(img: bytes, flags = cv.IMREAD_UNCHANGED) -> cv.Mat:
    arr = np.frombuffer(img, np.uint8)
    return cv.imdecode(arr, flags)

@R.curry
def b64encode(img: cv.Mat, format: str = ".png", url_safe: bool = True) -> str:
    """Encode `img` into a Base64 string"""
    _, encoded = cv.imencode(format, img)
    b64 = base64.urlsafe_b64encode(encoded) if url_safe else base64.b64encode(encoded)
    return b64.decode("utf-8")

def b64decode(b64img: str) -> cv.Mat:
    b = base64.urlsafe_b64decode(b64img)
    arr = np.frombuffer(b, np.uint8)
    return cv.imdecode(arr, -1)



import cv2 as cv
import numpy as np
import ramda as R

Img2D = cv.Mat
Img3D = cv.Mat
Img = Img2D
Mat3x3 = cv.Mat
_N = int
_2 = int

@R.curry
def imdecode(img, flags = cv.IMREAD_UNCHANGED) -> Img:
    return cv.imdecode(img, flags)

def imread(path: str) -> Img:
    """Read in RGB"""
    img = cv.imread(path)
    return cv.cvtColor(img, cv.COLOR_BGR2RGB)

def grayscale(img: Img) -> Img2D:
    """`Array[h, w, ...] -> Array[h, w]`"""
    return img if len(np.shape(img)) == 2 else cv.cvtColor(img, cv.COLOR_BGR2GRAY)

def threshold(img: Img) -> Img2D:
    """`Array[h, w, ...] -> Array[h, w]` with values in `{0, 255}` only"""
    gray_img = grayscale(img)
    return cv.adaptiveThreshold(~gray_img, 255, cv.ADAPTIVE_THRESH_MEAN_C, cv.THRESH_BINARY, 15, -3)

@R.curry
def cvtColor(src: Img, code: int) -> Img:
    return cv.cvtColor(src, code)

@R.curry
def dilate(src: Img, kernel: cv.Mat, iterations: int) -> Img:
    return cv.dilate(src, kernel=kernel, iterations=iterations)

@R.curry
def erode(src: Img, kernel: cv.Mat, iterations: int) -> Img:
    return cv.erode(src, kernel=kernel, iterations=iterations)